import errno
from send2trash.compat import PY3

if PY3:
    _permission_error = PermissionError  # noqa: F821
else:
    _permission_error = OSError


class TrashPermissionError(_permission_error):
    """A permission error specific to a trash directory.

    Raising this error indicates that permissions prevent us efficiently
    trashing a file, although we might still have permission to delete it.
    This is *not* used when permissions prevent removing the file itself:
    that will be raised as a regular PermissionError (OSError on Python 2).

    Application code that catches this may try to simply delete the file,
    or prompt the user to decide, or (on Freedesktop platforms), move it to
    'home trash' as a fallback. This last option probably involves copying the
    data between partitions, devices, or network drives, so we don't do it as
    a fallback.
    """

    def __init__(self, filename):
        _permission_error.__init__(self, errno.EACCES, "Permission denied", filename)



# Copyright 2017 Virgil Dupras

# This software is licensed under the "BSD" License as described in the "LICENSE" file,
# which should be included with this package. The terms are also available at
# http://www.hardcoded.net/licenses/bsd_license

from Foundation import NSFileManager, NSURL
from send2trash.compat import text_type
from send2trash.util import preprocess_paths


def check_op_result(op_result):
    # First value will be false on failure
    if not op_result[0]:
        # Error is in third value, localized failure reason matchs ctypes version
        raise OSError(op_result[2].localizedFailureReason())


def send2trash(paths):
    paths = preprocess_paths(paths)
    paths = [path.decode("utf-8") if not isinstance(path, text_type) else path for path in paths]
    for path in paths:
        file_url = NSURL.fileURLWithPath_(path)
        fm = NSFileManager.defaultManager()
        op_result = fm.trashItemAtURL_resultingItemURL_error_(file_url, None, None)
        check_op_result(op_result)
